import scanpy as sc
from pyscenic.aucell import aucell, derive_auc_threshold
from ctxcore.genesig import GeneSignature

def aucell_cal(adata, gmt, norm_type=False, adapt_signatures=True):
    """
    Perform AUCell-based enrichment scoring on an AnnData object.

    Parameters:
    -----------
    adata : AnnData
        Annotated data matrix.
    gmt : str
        Path to gene set in GMT format.
    norm_type : bool, optional
        Whether to normalize total counts (default: False).
    adapt_signatures : bool, optional
        Whether to filter gene sets to those with sufficient overlap (default: True).

    Returns:
    --------
    adata : AnnData
        AnnData object with AUCell scores added to `.obs`.
    """
    if norm_type:
        sc.pp.normalize_total(adata, target_sum=1e4)

    if adapt_signatures:
        geneset_dict = {}
        with open(gmt) as f:
            for line in f:
                data = line.strip().split('\t')
                geneset_dict[data[0]] = data[2:]

        gmt_tmp = f'{gmt}.tmp'
        with open(gmt_tmp, 'w') as f:
            for title, genes in geneset_dict.items():
                overlap_genes = list(set(genes) & set(adata.var_names))
                if len(overlap_genes) / len(genes) >= 0.5:
                    f.write('{0}\t{0}\t{1}\n'.format(title, '\t'.join(overlap_genes)))

        gs = GeneSignature.from_gmt(gmt_tmp, field_separator="\t", gene_separator="\t")
    else:
        gs = GeneSignature.from_gmt(gmt, field_separator="\t", gene_separator="\t")

    df = adata.to_df()
    percentiles = derive_auc_threshold(df)

    scores = aucell(
        exp_mtx=df,
        signatures=gs,
        auc_threshold=percentiles[0.01],
        seed=2,
        normalize=True,
        num_workers=1
    )

    for col in scores.columns:
        adata.obs[col] = scores.loc[adata.obs_names, col].tolist()

    return adata


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

d = {}
with open("scMalignantFinder/_version.py") as f:
    exec(f.read(), d)
    
setup(
    name='scMalignantFinder',
    version=d["__version__"],
    description='A tool for identitying malignant cells from single-cell RNA-seq data',
    long_description=long_description,
    author='Journey Yu',
    author_email='jonyyqn@mail.ustc.edu.cn',
    long_description_content_type="text/markdown",
    url="https://github.com/jonyyqn/scMalignantFinder",
    packages=find_packages(),
    python_requires='>=3.8',
    install_requires=[
        'scanpy==1.9.3',
        'anndata==0.9.1',
        'scikit-learn==1.2.2',
        'pyscenic==0.12.1',
        'dask[dataframe]==2024.11.2',
        'squidpy==1.3.1',
        'numpy>=1.24,<2',
        'joblib==1.4.2',
        'pandas==2.1.4',
        'matplotlib==3.6.3',
        'scipy==1.13.1',
        # 'numba==0.56.4',
        'h5py==3.7.0'
        
    ],
)
